

/**
 * Description of operand storage.
 * The Indirect fields exist because of arrays indexed with non-constants.
 *
 * if (IsIndirect) {
 *    value is at File[Index + IndirectFile[IndirectIndex]];
 *    // example GLSL:
 *    //   float a[10];
 *    //   int i;
 *    //   float x = a[i];   // a[i] is TEMP[aPos + TEMP[iPos]]
 *    // or:
 *    //   float a[10];
 *    //   uniform int u;
 *    //   float x = a[u];   // a[u] is TEMP[aPos + PARAM[uPos]]
 * }
 * else {
 *    value is at File[Index];
 *    // example GLSL:
 *    //   float x = y;      // y is TEMP[yPos]
 *    // or:
 *    //   float a[10];
 *    //   float x = a[2];   // y is TEMP[aPos + 2]
 * }
 *
 * When we do code generation, indirect storage references will be
 * implemented by setting the address register with the ARL instruction.
 */
struct storage {
   GLint File;
   GLint Index;
   GLboolean IsIndirect;
   GLint IndirectFile;
   GLint IndirectIndex;
};



storage *
emit_var(slang_ir_node *n)
{
   assert(n->Type == SLANG_IR_VAR);
   LookupVar;
   assert(file == PROGRAM_TEMPORARY || file == PROGRAM_PARAMETER);
   return storage(file, index, GL_FALSE, 0);
}


storage *
emit_const(slang_ir_node *n)
{
   index = _mesa_add_constant(n->value);

   return storage(PROGRAM_CONST, index, GL_FALSE, 0);
}


storage *
emit_element(slang_ir_node *n)
{
   storage *lhs_store, *rhs_store;
   const int elem_size = sizeof_type(n->Children[0]->type);

   lhs_store = emit(n->Children[0]);
   rhs_store = emit(n->Children[1]);

   if (elem_size != 1) {
      /* mul array index by array element size */
      tmp_store = alloc_tmp();
      inst_MUL(tmp_store, rhs_store, elem_size_store);
      rhs_store = tmp_store;
   }

   if (lhs_store->IsIndirect) {
      if (rhs_store->File == PROGRAM_CONSTANT) {
         /* the array index is a compile-time constant */
         return indirect_store(lhs_store->File,
                               lhs_store->Index + rhs_store->Value,
                               lhs_store->IndirectFile,
                               lhs_store->IndirectIndex);
      }
      else {
         /* need to add indexes */
         storage *tmp = alloc_temp();
         inst_ADD(tmp
                  TEMP[lhs_store->IndirectIndex]
                  TEMP[rhs_store->Index]);
         return indirect_store(lhs_store->File,
                               lhs_store->Index,
                               tmp->File,
                               tmp->Index);
      }
   }
   else {
      if (rhs_store->File == PROGRAM_CONSTANT) {
         /* the array index is a compile-time constant */
         return store(lhs_store->File,
                      lhs_store->Index + CONST[rhs_store->Index]);
      }
      else {
         assert(rhs_store->File == PROGRAM_TEMPORARY);
         return indirect_store(lhs_store->File,
                               lhs_store->Index,
                               rhs_store->File,
                               rhs_store->Index);
      }
   }

}

